#if defined(GL_ES)
precision GE_PRECISION float;
#else
#define lowp
#define mediump
#define highp
#endif

uniform mat4 ig_ModelViewMatrix;
uniform mat4 ig_ModelViewProjectionMatrix;
uniform mat4 ig_TextureMatrix;
uniform vec4 ig_LightDirectionInModelSpace;
uniform vec4 ig_env_atmosphere_custom_state_value;
uniform lowp vec4 ig_env_atmosphere_custom_color_value;

float saturate(float v) { return clamp(v, 0.0, 1.0); }
vec2 saturate(vec2 v) { return clamp(v, 0.0, 1.0); }
vec3 saturate(vec3 v) { return clamp(v, 0.0, 1.0); }
vec4 saturate(vec4 v) { return clamp(v, 0.0, 1.0); }

// Returns the quadrant portion of the given vertex flags.
float getQuadrant(vec4 vertexQuadrant) {
  return dot(vertexQuadrant, vec4(0, 1, 2, 3));
}

vec4 getMaskedVertex(vec4 pos, vec4 quadrantMask, vec4 vertexQuadrant) {
  return pos * dot(quadrantMask, vertexQuadrant);
}

// Computes a vertex color to show which quadrant a vertex is in.  Also marks
// vertices that would be rejected and error conditions (e.g. invalid
// quadrant).
vec4 getDebugVertexColor(vec4 quadrantMask, vec4 vertexQuadrant) {
  vec4 color;
  float quadrant = getQuadrant(vertexQuadrant);
  if (quadrant == 0.) {
    // Red
    color = vec4(1.0, 0.0, 0.0, 1.0);
  } else if (quadrant == 1.) {
    // Green
    color = vec4(0.0, 1.0, 0.0, 1.0);
  } else if (quadrant == 2.) {
    // Blue
    color = vec4(0.0, 0.0, 1.0, 1.0);
  } else if (quadrant == 3.) {
    // Cyan
    color = vec4(0.0, 1.0, 1.0, 1.0);
  } else {
    // Something went wrong, so flag verts in saturated Magenta.
    color = vec4(10.0, 0.0, 10.0, 1.0);
  }
  // Make hidden quadrants dimmer.
  if (dot(quadrantMask, vertexQuadrant) == 0.)
    color *= .5;
  return color;
}

#ifdef DEBUG_QUADRANT_COLORS
// Used by the pixel shader to show quadrants, culled verts and error
// conditions when DEBUG_QUADRANT_COLORS is defined.
varying vec4 vout_debug_color;
#endif

// When debugging is on, modulates the pixel color by the debug color output
// from the vertex shader.  Otherwise, the given pixel color is returned
// unchanged.
vec4 computeDebugPixelColor(vec4 pixelShaderColor) {
#ifdef DEBUG_QUADRANT_COLORS
  return pixelShaderColor * vout_debug_color;
#else
  return pixelShaderColor;
#endif
}
